-- BaseLogin procedure 

Use Admin

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[BaseLogin]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[BaseLogin]
GO


use Admin
go

CREATE PROCEDURE dbo.BaseLogin
AS
/*

create windows accounts on an instance based on [PROD-VM-DBA].Admin.api.PermissionDef table

NOTE: does not remove any logins. That would be done in Admin.dbo.ServiceRole

exec Admin.dbo.BaseLogin


1/28/14 Andre Quitta Created

6/9/14 Andre Quitta Only add instance level login if a database exists that it's associated to 
and its not an SERVER or CONNECT login, via @ConnectionCount
*/
begin
declare @SQL nvarchar(max)
declare @LoginName varchar(100)
declare @ConnectionCount int
set nocount on

IF OBJECT_ID('tempdb..#PermissionDef') IS NOT NULL
DROP TABLE #PermissionDef


-- from table of permissions
select distinct 
LoginName,
cast('' as varchar(100)) as PrincipalsName,
cast(0 as bit) as Done
into #PermissionDef
from [prod-vm-dba].admin.api.PermissionDef
where charindex(InstanceName,@@servername) > 0


-- match on what is already there
delete
from #PermissionDef 
where LoginName in (
select name
from master.sys.server_principals p
)


if (select count(*) from #PermissionDef) = 0
begin
print 'All Base Logins are current'
return
end


if (select count(*) from #PermissionDef where LoginName not like 'SW\%') > 0
begin

print 'Base Logins that are missing that are SQL Logins'

select * from #PermissionDef where LoginName not like 'SW\%'


delete 
from #PermissionDef 
where LoginName not like 'SW\%'
end



-- create the logins
while (select count(*) from #PermissionDef where Done = 0) > 0
begin
set @ConnectionCount = 0

select @LoginName = min(LoginName)
from #PermissionDef
where Done = 0

set @ConnectionCount = 0 
+ (select count(*) from [prod-vm-dba].admin.api.PermissionDef where charindex(InstanceName,@@servername) > 0 and LoginName = @LoginName and DBName in (select name from master.sys.databases) )
+ (select count(*) from [prod-vm-dba].admin.api.PermissionDef where charindex(InstanceName,@@servername) > 0 and LoginName = @LoginName and PermissionType in ('Connect', 'Server') )

--print '@LoginName = '+ @LoginName + ' , @ConnectionCount = ' + convert(varchar(10), @ConnectionCount)

if @ConnectionCount > 0
begin  
set @SQL = '
CREATE LOGIN ['+ @LoginName +'] from windows
'
print @SQL
exec sp_executeSQL @SQL
end -- if @ConnectionCount > 0
else
print '                   Didnt create login ' + @LoginName + ' because there are no databases on ' + @@Servername + ' that use it'

update #PermissionDef
set Done = 1
where LoginName = @LoginName
end -- while (select count(*) from #PermissionDef where Done = 0) > 0
end



-- ServiceRole procedure 

create procedure dbo.ServiceRole
as
begin
/*
exec Admin.dbo.ServiceRole
Description
   grant service account permissions in non-Production environments
Used by          SQL job SecurityAutoFix
Dependent on         DBAInstance.Admin.dbo.BaseLogin

Outline
-- check if database exists
-- pull all other rights in that database
-- associated login with role or GRANTs
*/
declare @SQL nvarchar(max)
declare @DBName varchar(100)
declare @SchemaName varchar(20)
declare @LoginName varchar(100)
declare @PermissionType varchar(15)
declare @RoleName varchar(50)
declare @ObjectName varchar(100)
declare @MinRowNum int
declare @Version varchar(5)
set nocount on

   exec Admin.dbo.BaseLogin

   -- SQL version, either '10' or '11'
   select @Version = left(convert(varchar(20),SERVERPROPERTY ('ProductVersion')),2)

   select ROW_NUMBER() OVER (Order by DBName, LoginName, RoleName) AS RowNumber,
          *,
          cast(0 as bit) as Done
   into   #PermissionDef
   from   [dba_instance].admin.api.PermissionDef
   where  charindex(InstanceName,@@servername) > 0

   while (select count(*) from #PermissionDef where Done = 0) > 0
   begin
       select @MinRowNum = min(RowNumber)
       from   #PermissionDef
       where  Done = 0

       select @DBName = DBName,
          @SchemaName = SchemaName,
          @LoginName = LoginName,
          @PermissionType = PermissionType,
          @RoleName = RoleName,
          @ObjectName = ObjectName
       from   #PermissionDef
       where  RowNumber = @MinRowNum
--print 'acting on DBName [' + @DBName + ']  , ' + @LoginName + '/' + @RoleName + ' for PermissionType ' + @PermissionType

       if (select count(*) from master.sys.databases where name = @DBName) = 1
       begin
          if @PermissionType = 'Role'
              set @SQL = '

                 use [' + @DBName + ']
                 if (select count(*) from sys.database_principals where name = ''' + @RoleName + ''' and type = ''R'' and owning_principal_id is not null) > 0
                 begin
                     if (select count(*) from  sysusers where name = '''+ @LoginName +''') = 0
                        create user  ['+ @LoginName +'] 
                     exec sp_addrolemember N'''+ @RoleName +''', '''+ @LoginName +'''          
                 end
                 else
                     print ''DBName [' + @DBName + ']    ' + @LoginName + ' / ' + @RoleName + ' cannot be used because of a missing Role or the Login is not on the Instance''
              '
          else if @PermissionType = 'Grant' and coalesce(@ObjectName, '') = ''
              set @SQL = '

                 use [' + @DBName + ']
                 if not exists (select * from  sysusers where name = '''+ @LoginName +''')
                     create USER  ['+ @LoginName +'] 
                 grant '+ @RoleName +' on schema :: '+ @SchemaName +' to ['+ @LoginName +']
              '
          else if @PermissionType = 'Grant' and coalesce(@ObjectName, '') <> ''
              set @SQL = '

                 use [' + @DBName + ']
                 if not exists (select * from  sysusers where name = '''+ @LoginName +''')
                     create USER  ['+ @LoginName +'] 
                 grant '+ @RoleName +' on '+ @ObjectName +' to ['+ @LoginName +']
              '
          else if @PermissionType = 'Member'
              set @SQL = '

                 use [' + @DBName + ']
                 if not exists (select * from  sysusers where name = '''+ @LoginName +''')
                       create USER  ['+ @LoginName +'] 
                 EXEC sp_addrolemember N'''+ @RoleName +''', ['+ @LoginName +']
              '
          else if @PermissionType = 'Server' and @RoleName = 'Connect'
              set @SQL = '

                 use master
                
                 if not exists (select * from  sysusers where name = '''+ @LoginName +''')
                       create USER  ['+ @LoginName +'] 
                 GRANT CONNECT SQL TO ['+ @LoginName +'];
              '
          else if @PermissionType = 'Server' and @Version = '10'                  -- SQL 2008
              set @SQL = '

                 use master
                

                 if not exists (select * from  sysusers where name = '''+ @LoginName +''')
                       create USER  ['+ @LoginName +'] 
                 EXEC master..sp_addsrvrolemember @loginame = N'''+ @LoginName +''', @rolename = N'''+ @RoleName +'''
                 --EXEC sp_addrolemember N'''+ @RoleName +''', ['+ @LoginName +']
              '

          else if @PermissionType = 'Server' and @Version = '11'                  -- SQL 2012
              set @SQL = '

                 use master
                
                 if not exists (select * from  sysusers where name = '''+ @LoginName +''')
                       create USER  ['+ @LoginName +'] 
                 ALTER SERVER ROLE '+ @RoleName + ' ADD MEMBER [' + @LoginName + ']
              '

          else
              print 'NO CRITERIA MATCH ??'
          print @SQL
          exec sp_executeSQL @SQL
       end
       else
          print 'DBName [' + @DBName + ']  does not exist, so ' + @LoginName + '/' + @RoleName + ' cannot be used'

       update #PermissionDef
       set    Done = 1
       where  RowNumber = @MinRowNum
   end -- while (select count(*) from #PermissionDef where Done = 0) > 0

end        
